@@ -11,15 +11,20 @@ import operator
 import types
 
 import six
+if six.PY3:
+    from .compat import is_bound_method as ismethod
+else:
+    from inspect import ismethod
 
 from webob import (Request as WebObRequest, Response as WebObResponse, exc,
                    acceptparse)
+from webob.multidict import NestedMultiDict
 
 from .compat import urlparse, unquote_plus, izip
 from .secure import handle_security
 from .templating import RendererFactory
 from .routing import lookup_controller, NonCanonicalPath
-from .util import _cfg, encode_if_needed
+from .util import _cfg, encode_if_needed, getargspec
 from .middleware.recursive import ForwardRequestException
 
 
@@ -254,7 +259,8 @@ class PecanBase(object):
             module = __import__(name, fromlist=fromlist)
             kallable = getattr(module, parts[-1])
             msg = "%s does not represent a callable class or function."
-            assert hasattr(kallable, '__call__'), msg % item
+            if not six.callable(kallable):
+                raise TypeError(msg % item)
             return kallable()
 
         raise ImportError('No item named %s' % item)
@@ -499,6 +505,14 @@ class PecanBase(object):
         # fetch any parameters
         if req.method == 'GET':
             params = dict(req.GET)
+        elif req.content_type in ('application/json',
+                                  'application/javascript'):
+            try:
+                if not isinstance(req.json, dict):
+                    raise TypeError('%s is not a dict' % req.json)
+                params = dict(NestedMultiDict(req.GET, req.json))
+            except (TypeError, ValueError):
+                params = dict(req.params)
         else:
             params = dict(req.params)
 
@@ -527,6 +541,15 @@ class PecanBase(object):
         resp = state.response
         pecan_state = req.pecan
 
+        # If a keyword is supplied via HTTP GET or POST arguments, but the
+        # function signature does not allow it, just drop it (rather than
+        # generating a TypeError).
+        argspec = getargspec(controller)
+        keys = kwargs.keys()
+        for key in keys:
+            if key not in argspec.args and not argspec.keywords:
+                kwargs.pop(key)
+
         # get the result from the controller
         result = controller(*args, **kwargs)
 
@@ -594,9 +617,15 @@ class PecanBase(object):
             else:
                 text = None
                 if state.response.charset:
-                    # `response.text` cannot be accessed without a charset
-                    # (because we don't know which encoding to use)
-                    text = state.response.text
+                    # `response.text` cannot be accessed without a valid
+                    # charset (because we don't know which encoding to use)
+                    try:
+                        text = state.response.text
+                    except UnicodeDecodeError:
+                        # If a valid charset is not specified, don't bother
+                        # trying to guess it (because there's obviously
+                        # content, so we know this shouldn't be a 204)
+                        pass
                 if not any((state.response.body, text)):
                     state.response.status = 204
 
@@ -684,15 +713,23 @@ class ExplicitPecan(PecanBase):
         # When comparing the argspec of the method to GET/POST params,
         # ignore the implicit (req, resp) at the beginning of the function
         # signature
-        signature_error = TypeError(
-            'When `use_context_locals` is `False`, pecan passes an explicit '
-            'reference to the request and response as the first two arguments '
-            'to the controller.\nChange the `%s.%s.%s` signature to accept '
-            'exactly 2 initial arguments (req, resp)' % (
+        if hasattr(state.controller, '__self__'):
+            _repr = '.'.join((
                 state.controller.__self__.__class__.__module__,
                 state.controller.__self__.__class__.__name__,
                 state.controller.__name__
-            )
+            ))
+        else:
+            _repr = '.'.join((
+                state.controller.__module__,
+                state.controller.__name__
+            ))
+
+        signature_error = TypeError(
+            'When `use_context_locals` is `False`, pecan passes an explicit '
+            'reference to the request and response as the first two arguments '
+            'to the controller.\nChange the `%s` signature to accept exactly '
+            '2 initial arguments (req, resp)' % _repr
         )
         try:
             positional = argspec.args[:]
@@ -705,7 +742,13 @@ class ExplicitPecan(PecanBase):
         args, varargs, kwargs = super(ExplicitPecan, self).get_args(
             state, all_params, remainder, argspec, im_self
         )
-        args = [state.request, state.response] + args
+
+        if ismethod(state.controller):
+            args = [state.request, state.response] + args
+        else:
+            # generic controllers have an explicit self *first*
+            # (because they're decorated functions, not instance methods)
+            args[1:1] = [state.request, state.response]
         return args, varargs, kwargs
 
 
