@@ -1,4 +1,4 @@
-from inspect import getargspec, ismethod
+from inspect import ismethod
 import warnings
 
 from webob import exc
@@ -7,7 +7,7 @@ import six
 from .core import abort
 from .decorators import expose
 from .routing import lookup_controller, handle_lookup_traversal
-from .util import iscontroller
+from .util import iscontroller, getargspec
 
 
 class RestController(object):
@@ -54,7 +54,10 @@ class RestController(object):
             request.pecan.get('routing_args', [])
         )
         if len(remainder) < fixed_args:
-            abort(400)
+            # For controllers that are missing intermediate IDs
+            # (e.g., /authors/books vs /authors/1/books), return a 404 for an
+            # invalid path.
+            abort(404)
 
     @expose()
     def _route(self, args, request=None):
