@@ -44,6 +44,34 @@ class TestEmptyContent(PecanTestCase):
             def index(self):
                 pass
 
+            @expose()
+            def explicit_body(self):
+                response.body = b_('Hello, World!')
+
+            @expose()
+            def empty_body(self):
+                response.body = b_('')
+
+            @expose()
+            def explicit_text(self):
+                response.text = six.text_type('Hello, World!')
+
+            @expose()
+            def empty_text(self):
+                response.text = six.text_type('')
+
+            @expose()
+            def explicit_json(self):
+                response.json = {'foo': 'bar'}
+
+            @expose()
+            def explicit_json_body(self):
+                response.json_body = {'foo': 'bar'}
+
+            @expose()
+            def non_unicode(self):
+                return chr(0xc0)
+
         return TestApp(Pecan(RootController()))
 
     def test_empty_index(self):
@@ -53,6 +81,69 @@ class TestEmptyContent(PecanTestCase):
         self.assertEqual(r.headers['Content-Length'], '0')
         self.assertEqual(len(r.body), 0)
 
+    def test_index_with_non_unicode(self):
+        r = self.app_.get('/non_unicode/')
+        self.assertEqual(r.status_int, 200)
+
+    def test_explicit_body(self):
+        r = self.app_.get('/explicit_body/')
+        self.assertEqual(r.status_int, 200)
+        self.assertEqual(r.body, b_('Hello, World!'))
+
+    def test_empty_body(self):
+        r = self.app_.get('/empty_body/')
+        self.assertEqual(r.status_int, 204)
+        self.assertEqual(r.body, b_(''))
+
+    def test_explicit_text(self):
+        r = self.app_.get('/explicit_text/')
+        self.assertEqual(r.status_int, 200)
+        self.assertEqual(r.body, b_('Hello, World!'))
+
+    def test_empty_text(self):
+        r = self.app_.get('/empty_text/')
+        self.assertEqual(r.status_int, 204)
+        self.assertEqual(r.body, b_(''))
+
+    def test_explicit_json(self):
+        r = self.app_.get('/explicit_json/')
+        self.assertEqual(r.status_int, 200)
+        json_resp = json.loads(r.body.decode())
+        assert json_resp == {'foo': 'bar'}
+
+    def test_explicit_json_body(self):
+        r = self.app_.get('/explicit_json_body/')
+        self.assertEqual(r.status_int, 200)
+        json_resp = json.loads(r.body.decode())
+        assert json_resp == {'foo': 'bar'}
+
+
+class TestAppIterFile(PecanTestCase):
+    @property
+    def app_(self):
+        class RootController(object):
+            @expose()
+            def index(self):
+                body = six.BytesIO(b_('Hello, World!'))
+                response.body_file = body
+
+            @expose()
+            def empty(self):
+                body = six.BytesIO(b_(''))
+                response.body_file = body
+
+        return TestApp(Pecan(RootController()))
+
+    def test_body_generator(self):
+        r = self.app_.get('/')
+        self.assertEqual(r.status_int, 200)
+        assert r.body == b_('Hello, World!')
+
+    def test_empty_body_generator(self):
+        r = self.app_.get('/empty')
+        self.assertEqual(r.status_int, 204)
+        assert len(r.body) == 0
+
 
 class TestIndexRouting(PecanTestCase):
 
@@ -349,6 +440,16 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('index: 4')
 
+    def test_explicit_json_kwargs(self):
+        r = self.app_.post_json('/', {'id': '4'})
+        assert r.status_int == 200
+        assert r.body == b_('index: 4')
+
+    def test_path_with_explicit_json_kwargs(self):
+        r = self.app_.post_json('/4', {'id': 'four'})
+        assert r.status_int == 200
+        assert r.body == b_('index: 4')
+
     def test_multiple_kwargs(self):
         r = self.app_.get('/?id=5&dummy=dummy')
         assert r.status_int == 200
@@ -359,6 +460,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('index: 6')
 
+    def test_json_kwargs_from_root(self):
+        r = self.app_.post_json('/', {'id': '6', 'dummy': 'dummy'})
+        assert r.status_int == 200
+        assert r.body == b_('index: 6')
+
         # multiple args
 
     def test_multiple_positional_arguments(self):
@@ -386,6 +492,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('multiple: five, six')
 
+    def test_positional_args_with_json_kwargs(self):
+        r = self.app_.post_json('/multiple', {'one': 'five', 'two': 'six'})
+        assert r.status_int == 200
+        assert r.body == b_('multiple: five, six')
+
     def test_positional_args_with_url_encoded_dictionary_kwargs(self):
         r = self.app_.post('/multiple', {'one': 'Five%20', 'two': 'Six%20%21'})
         assert r.status_int == 200
@@ -436,6 +547,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('optional: 4')
 
+    def test_optional_arg_with_json_kwargs(self):
+        r = self.app_.post_json('/optional', {'id': '4'})
+        assert r.status_int == 200
+        assert r.body == b_('optional: 4')
+
     def test_optional_arg_with_url_encoded_kwargs(self):
         r = self.app_.post('/optional', {'id': 'Some%20Number'})
         assert r.status_int == 200
@@ -446,6 +562,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('optional: 5')
 
+    def test_multiple_positional_arguments_with_json_kwargs(self):
+        r = self.app_.post_json('/optional/5', {'id': 'five'})
+        assert r.status_int == 200
+        assert r.body == b_('optional: 5')
+
     def test_multiple_positional_url_encoded_arguments_with_kwargs(self):
         r = self.app_.post('/optional/Some%20Number', {'id': 'five'})
         assert r.status_int == 200
@@ -466,6 +587,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('optional: 7')
 
+    def test_optional_arg_with_multiple_json_kwargs(self):
+        r = self.app_.post_json('/optional', {'id': '7', 'dummy': 'dummy'})
+        assert r.status_int == 200
+        assert r.body == b_('optional: 7')
+
     def test_optional_arg_with_multiple_url_encoded_dictionary_kwargs(self):
         r = self.app_.post('/optional', {
             'id': 'Some%20Number',
@@ -530,6 +656,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('multiple_optional: 1, None, None')
 
+    def test_multiple_optional_positional_args_with_json_kwargs(self):
+        r = self.app_.post_json('/multiple_optional', {'one': '1'})
+        assert r.status_int == 200
+        assert r.body == b_('multiple_optional: 1, None, None')
+
     def test_multiple_optional_positional_args_with_encoded_dict_kwargs(self):
         r = self.app_.post('/multiple_optional', {'one': 'One%21'})
         assert r.status_int == 200
@@ -540,6 +671,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('multiple_optional: 1, None, None')
 
+    def test_multiple_optional_positional_args_and_json_kwargs(self):
+        r = self.app_.post_json('/multiple_optional/1', {'one': 'one'})
+        assert r.status_int == 200
+        assert r.body == b_('multiple_optional: 1, None, None')
+
     def test_multiple_optional_encoded_positional_args_and_dict_kwargs(self):
         r = self.app_.post('/multiple_optional/One%21', {'one': 'one'})
         assert r.status_int == 200
@@ -565,6 +701,14 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('multiple_optional: 1, 2, 3')
 
+    def test_multiple_optional_args_with_multiple_json_kwargs(self):
+        r = self.app_.post_json(
+            '/multiple_optional',
+            {'one': '1', 'two': '2', 'three': '3', 'four': '4'}
+        )
+        assert r.status_int == 200
+        assert r.body == b_('multiple_optional: 1, 2, 3')
+
     def test_multiple_optional_args_with_multiple_encoded_dict_kwargs(self):
         r = self.app_.post(
             '/multiple_optional',
@@ -618,6 +762,14 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('variable_args: ')
 
+    def test_variable_args_with_json_kwargs(self):
+        r = self.app_.post_json(
+            '/variable_args',
+            {'id': '3', 'dummy': 'dummy'}
+        )
+        assert r.status_int == 200
+        assert r.body == b_('variable_args: ')
+
     def test_variable_kwargs(self):
         r = self.app_.get('/variable_kwargs')
         assert r.status_int == 200
@@ -644,6 +796,14 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('variable_kwargs: dummy=dummy, id=3')
 
+    def test_multiple_variable_kwargs_with_json_kwargs(self):
+        r = self.app_.post_json(
+            '/variable_kwargs',
+            {'id': '3', 'dummy': 'dummy'}
+        )
+        assert r.status_int == 200
+        assert r.body == b_('variable_kwargs: dummy=dummy, id=3')
+
     def test_multiple_variable_kwargs_with_encoded_dict_kwargs(self):
         r = self.app_.post(
             '/variable_kwargs',
@@ -688,6 +848,14 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('variable_all: 6, day=12, month=1')
 
+    def test_variable_post_with_json_kwargs(self):
+        r = self.app_.post_json(
+            '/variable_all/6',
+            {'month': '1', 'day': '12'}
+        )
+        assert r.status_int == 200
+        assert r.body == b_('variable_all: 6, day=12, month=1')
+
     def test_variable_post_mixed(self):
         r = self.app_.post(
             '/variable_all/7',
@@ -696,6 +864,14 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('variable_all: 7, day=12, id=seven, month=1')
 
+    def test_variable_post_mixed_with_json(self):
+        r = self.app_.post_json(
+            '/variable_all/7',
+            {'id': 'seven', 'month': '1', 'day': '12'}
+        )
+        assert r.status_int == 200
+        assert r.body == b_('variable_all: 7, day=12, id=seven, month=1')
+
     def test_no_remainder(self):
         try:
             r = self.app_.get('/eater')
@@ -752,6 +928,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('eater: 9, None, day=12, month=1')
 
+    def test_post_remainder_with_json_kwargs(self):
+        r = self.app_.post_json('/eater/9', {'month': '1', 'day': '12'})
+        assert r.status_int == 200
+        assert r.body == b_('eater: 9, None, day=12, month=1')
+
     def test_post_many_remainders_with_many_kwargs(self):
         r = self.app_.post(
             '/eater/10',
@@ -760,6 +941,14 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('eater: 10, dummy, day=12, month=1')
 
+    def test_post_many_remainders_with_many_json_kwargs(self):
+        r = self.app_.post_json(
+            '/eater/10',
+            {'id': 'ten', 'month': '1', 'day': '12', 'dummy': 'dummy'}
+        )
+        assert r.status_int == 200
+        assert r.body == b_('eater: 10, dummy, day=12, month=1')
+
 
 class TestDefaultErrorRendering(PecanTestCase):
 
